import static es.urjc.etsii.code.concurrency.SimpleConcurrent.*;
import es.urjc.etsii.code.concurrency.SimpleSemaphore;

public class Simulador {

	private static final int N = 3; // Numero de gatos
	private static final int M = 2; // Numero de voluntarios
	private static Dispensador dispensador; // Dispensador de comida
	
	// Para la sincronización de barrera cíclica de los gatos
	private static int nProcesos = 0;
	private static SimpleSemaphore emNProcesos = new SimpleSemaphore(1); // Para el acceso concurrente a nProcesos
	private static SimpleSemaphore esperaGatos = new SimpleSemaphore(0); // Para la espera de un gato a los demás
	private static SimpleSemaphore desbloqueo = new SimpleSemaphore(0); // Para el desbloqueo correcto de los gatos

	public static void gato() {
		while (true) {
			
			comerGato();
			
			// Sincronización de barrera
			emNProcesos.acquire();
			nProcesos++;
			if (nProcesos < N) {
				emNProcesos.release();
				vivirGato();
				esperaGatos.acquire();
				desbloqueo.release();
			} else {
				nProcesos = 0;
				printlnI("Todos los gatos pueden volver a comer");
				esperaGatos.release(N-1);
				desbloqueo.acquire(N-1);				
				emNProcesos.release();
				vivirGato();
			}
		}
	}

	public static void voluntario() {
		while (true) {
			llevarComida();
			vivirVoluntario();
		}
	}

	public static void comerGato() {
		printlnI("GLlega");
		dispensador.sacarComida();
		printlnI("GEmpieza"); // Empieza a comer
		sleepRandom(5000);
		printlnI("GTermina"); // Termina de comer
	}

	public static void vivirGato() {
		sleepRandom(5000);
		printlnI("GVive");
		sleepRandom(5000);
	}

	public static void llevarComida() {
		printlnI("VLlega");
		sleepRandom(5000);
		dispensador.introducirComida();
		printlnI("VSeVa");
	}

	public static void vivirVoluntario() {
		sleepRandom(5000);
		printlnI("VVive");
		sleepRandom(5000);
	}

	public static void main(String[] args) {

		dispensador = new Dispensador();

		// Creamos los hilos de los gatos y voluntarios y los iniciamos
		createThreads(N, "gato");
		createThreads(M, "voluntario");
		startThreadsAndWait();
	}

}
